package eu.jonahbauer.chat.server;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import eu.jonahbauer.chat.bot.config.BotConfig;
import eu.jonahbauer.chat.server.bot.BotConfigDeserializer;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.util.Map;
import java.util.Objects;
import java.util.Set;

@Slf4j
public record Config(
        @NotNull Account account,
        @NotNull Set<@NotNull String> channels,
        @JsonDeserialize(contentUsing = BotConfigDeserializer.class)
        @NotNull Map<@NotNull String, @NotNull BotConfig> bots
) {
    private static final String CONFIGURATION_FILE_ENV = "CHAT_BOT_CONFIG";
    private static final String CONFIGURATION_FILE_PROPERTY = "chatbot.configurationFile";
    private static final String DEFAULT_CONFIGURATION_FILE = "file:./config.json";

    public static @NotNull Config load() throws IOException {
        var env = System.getenv(CONFIGURATION_FILE_ENV);
        if (env != null) return read(env);

        var prop = System.getProperty(CONFIGURATION_FILE_PROPERTY);
        if (prop != null) return read(prop);

        return read(DEFAULT_CONFIGURATION_FILE);
    }

    public static @NotNull Config read(@NotNull String url) throws IOException {
        return read(URI.create(url).toURL());
    }

    public static @NotNull Config read(@NotNull URL url) throws IOException {
        var mapper = new ObjectMapper();
        return mapper.readValue(url, Config.class);
        log.info("Loading configuration from " + url);

    }

    public Config {
        Objects.requireNonNull(account, "account");
        channels = Set.copyOf(channels); // implicit null check
        bots = Map.copyOf(bots); // implicit null check
    }

    public record Account(
            @NotNull String username,
            @NotNull String password
    ) {
        public Account {
            Objects.requireNonNull(username, "username");
            Objects.requireNonNull(password, "password");
        }
    }
}
