package eu.jonahbauer.chat.server.bot;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonToken;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
import eu.jonahbauer.chat.bot.config.BotConfig;
import eu.jonahbauer.chat.bot.config.BotConfigurationException;

import java.io.IOException;
import java.util.ArrayList;

public class BotConfigDeserializer extends StdDeserializer<BotConfig> {

    protected BotConfigDeserializer() {
        super(BotConfig.class);
    }

    @Override
    public BotConfig deserialize(JsonParser p, DeserializationContext ctxt) throws IOException {
        if (p.currentToken() == JsonToken.START_OBJECT) {
            p.nextToken();
        }

        var builder = BotConfig.builder();
        while (p.currentToken() == JsonToken.FIELD_NAME) {
            var key = p.currentName();

            switch (p.nextToken()) {
                case VALUE_STRING -> builder.value(key, p.getValueAsString());
                case VALUE_FALSE -> builder.value(key, false);
                case VALUE_TRUE -> builder.value(key, true);
                case VALUE_NULL -> {}
                case VALUE_NUMBER_FLOAT -> builder.value(key, p.getDoubleValue());
                case VALUE_NUMBER_INT -> builder.value(key, p.getLongValue());
                case START_ARRAY -> {
                    var list = new ArrayList<String>();
                    while (p.nextToken() != JsonToken.END_ARRAY) {
                        if (p.currentToken() == JsonToken.VALUE_STRING) {
                            list.add(p.getValueAsString());
                        } else {
                            throw new BotConfigurationException("Unsupported property type in array.");
                        }
                    }
                    builder.value(key, list);
                }
                case START_OBJECT -> throw new BotConfigurationException("Unsupported property type: object");
                default -> throw new BotConfigurationException("Unsupported property property type.");
            }

            p.nextToken();
        }

        if (p.currentToken() == JsonToken.END_OBJECT) {
            return builder.build();
        }

        throw new JsonParseException(p, "Unexpected token: " + p.currentToken(), p.currentLocation());
    }
}
